#!/usr/bin/env python3

import sys, math
import numpy

usage = "%s -un or -semi (color1: g-i) (color2: i-z) (color3: griz)\n e.g., classify_object_with_npy_parameter_SDSS.py -semi 0.28 -0.06 0.69\n e.g., classify_object_with_npy_parameter_SDSS.py -un 0.28 -0.06 0.69" % (sys.argv[0])

method = None
try:
    if sys.argv[1] == "-un":
        method = "unsupervised"
    elif sys.argv[1] == "-semi":
        method = "semi-supervised"
    color1 = float(sys.argv[2])
    color2 = float(sys.argv[3])
    color3 = float(sys.argv[4])
    use_color = numpy.array([color1, color2, color3])
except:
    print(usage)
    sys.exit(1)

if method is None:
    print(usage)
    sys.exit(1)

# constants
use_factor_constant = 1.0 / math.pow(2.0*math.pi, 1.5)
n_dim = 3

if method == "unsupervised":
    in_npy_fn = "unsupervised_parameters_SDSS.npy"
    # classes from Roh et al. 2020
    use_cluster_dict = {2: 'S', 1: 'C', 6: 'V', 4: 'X'}
else:
    in_npy_fn = "semi_supervised_parameters_SDSS.npy"
    # classes from Roh et al. 2020
    use_cluster_dict = {3: 'C', 4: 'D', 5: 'K', 6: 'L', 9: 'S', 11: 'V', 12 : 'X'}
#print('use_cluster_dict ->', use_cluster_dict)
#print("... reading %s" % (in_npy_fn))
try:
    in_npy_fd = open(in_npy_fn, 'rb')
except:
    print("[ERROR] opening %s failed." % (in_npy_fn))
    sys.exit(1)
pi_array = numpy.load(in_npy_fd)
mu_array = numpy.load(in_npy_fd)
sigma_array = numpy.load(in_npy_fd)
in_npy_fd.close()

num_mixtures = pi_array.size

class_probability_list = []
max_class_probability = -1.0
max_class_cluster_id = -1
for use_cluster_id in range(1, num_mixtures+1):
    use_row_mu = use_cluster_id - 1
    use_pi = pi_array[use_row_mu]
    use_mu_array = mu_array[use_row_mu, :]
    use_row_sigma = (use_cluster_id - 1) * n_dim
    use_sigma_array = \
    numpy.matrix(sigma_array[use_row_sigma:(use_row_sigma + n_dim), :])
    use_factor_sigma = 1.0 / math.sqrt(numpy.linalg.det(use_sigma_array))
    use_factor = use_factor_constant * use_factor_sigma
    use_inverse_sigma_array = numpy.linalg.inv(use_sigma_array)
    use_term = use_color - use_mu_array
    D_M_sq = numpy.dot(numpy.dot(use_term, use_inverse_sigma_array), \
    use_term.T)
    D_M_sq = float(D_M_sq)
    class_probability_raw = use_factor * math.exp(-0.5*D_M_sq)
    class_probability = use_pi * class_probability_raw
    if class_probability > max_class_probability:
        max_class_cluster_id = use_cluster_id
        max_class_probability = class_probability
    class_probability_list.append(class_probability)
try:
    class_str = use_cluster_dict[max_class_cluster_id]
except:
    class_str = "NONE"
sum_of_class_probability = sum(class_probability_list)
out_str = ""
for ind in range(0, len(class_probability_list)):
    use_class_probability = class_probability_list[ind] / sum_of_class_probability
    try:
        use_class_str = use_cluster_dict[ind+1]
    except:
        use_class_str = '-'
    out_str = out_str + "%s %g " % (use_class_str, use_class_probability)
print(out_str.strip())
